import { useState } from 'react'
import { Button, Flex, Modal, Select, Form, Input, Popover, Card } from 'antd'
import { QuestionCircleOutlined } from '@ant-design/icons'
import AppProgressSteps from '@/components/AppProgressSteps'
import { useDataFieldQuery } from '@/hooks/useQuery'
import { useAI } from '@/ai/useAI'
import MarkdownRender from '@/components/MarkdownRender'

export default function ButtonModal() {
  const { fieldNameOptions } = useDataFieldQuery()

  const [open, setOpen] = useState(false)
  const [current, setCurrent] = useState(0)
  const [form] = Form.useForm()

  const { post, isLoading } = useAI()

  // AI设置
  const handleAISet = async () => {
    await form.validateFields(['independent_variables', 'dependent_variable'])

    const values = form.getFieldsValue()
    const message = await post({
      response_format: { type: 'json_object' },
      messages: [
        {
          role: 'user',
          content: `根据提供的数据集，设计一个Apriori算法，返回JSON格式。
          特征列：${values.independent_variables?.join(', ')}
返回格式示例：
json
        {
          "results": { "min_support": "最小支持度（min_support）", "min_confidence": "最小置信度（min_confidence）", "min_lift": "最小提升度（min_lift）", "min_length": "规则最小长度（min_length）", "max_length": "规则最大长度（max_length）", "metric": "评估指标（metric）", "min_threshold": "最小阈值（min_threshold）"}
        }
          `,
        },
      ],
    })
    console.log(message.json)
    if (message.json && message.json.results) {
      form.setFieldsValue(message.json.results)
    }
  }

  // AI计算
  const [results, setResults] = useState<any>({})
  const code = `${results.code?.replace(/\\n/gi, '\n')}`
  const [openResults, setOpenResults] = useState(false)
  const handleAICompute = async () => {
    const values = await form.validateFields()
    const message = await post({
      response_format: { type: 'json_object' },
      messages: [
        {
          role: 'user',
          content: `根据提供的数据集，设置的Apriori算法进行试算检查，返回JSON格式。
          特征列：${values.independent_variables?.join(', ')}
          最小支持度（min_support）：${values.min_support}
          最小置信度（min_confidence）：${values.min_confidence}
          最小提升度（min_lift）：${values.min_lift}
          规则最小长度（min_length）：${values.min_length}
          规则最大长度（max_length）：${values.max_length}
          评估指标（metric）：${values.metric}
          最小阈值（min_threshold）：${values.min_threshold}
          返回格式示例：
          json
          {
           results: { pass: 'true或者false，代表通过不通过', code: "Python运行的代码，markdown能解析的代码", result: "计算结果为纯文本形式" },
          }
           `,
        },
      ],
    })
    console.log(message.json)
    if (message.json && message.json.results) {
      setResults(message.json.results)
    }
    if (current === 1) {
      setStep(3)
    }
  }

  const [step, setStep] = useState<number>(-1)

  // 处理下一步按钮逻辑
  const handleNext = async () => {
    await form.validateFields()
    setCurrent(current + 1)
    setStep(-1)
  }

  // 开始处理
  const handleStart = () => {
    setStep(1)
    handleAICompute()
  }

  // 关闭并重置
  const handleClose = () => {
    setOpen(false)
    setCurrent(0)
    setStep(-1)
  }

  const steps = [
    {
      title: '设置算法',
      content: (
        <>
          <Form.Item label="请选择特征列" name="independent_variables" rules={[{ required: true, message: '请选择' }]}>
            <Select options={fieldNameOptions} placeholder="请选择" mode="multiple" optionFilterProp="label" />
          </Form.Item>

          <Flex gap={20}>
            <div style={{ flex: 1 }}>
              <Form.Item label="请输入最小支持度（min_support）">
                <Flex gap={10}>
                  <Form.Item name="min_support" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="最小支持度"
                    content={
                      <>
                        控制模型是否包含截距项（即偏置项，用于调整模型基线）
                        <br />
                        取值范围：
                        <br />
                        True（模型更灵活，适合大多数场景）
                        <br />
                        False（模型强制通过原点，可能欠拟合）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入最小置信度（min_confidence）">
                <Flex gap={10}>
                  <Form.Item name="min_confidence" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="最小置信度"
                    content={
                      <>
                        决定是否复制输入数据（避免修改原始数据）
                        <br />
                        取值范围：
                        <br />
                        True（安全但内存占用高）
                        <br />
                        False（节省内存，但可能覆盖原始数据）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入最小提升度（min_lift）">
                <Flex gap={10}>
                  <Form.Item name="min_lift" noStyle rules={[{ required: true, message: '请输入' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="最小提升度"
                    content={
                      <>
                        选择优化算法（不同算法支持不同正则化类型和计算效率）
                        <br />
                        取值范围：
                        <br />
                        None（单线程）或 ≥1 的整数（如 -1 表示使用所有核心）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入规则最小长度（min_length）">
                <Flex gap={10}>
                  <Form.Item name="min_length" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="规则最小长度"
                    content={
                      <>
                        是否强制回归系数为非负值（适用于需正向解释的场景，如经济学或物理模型）
                        <br />
                        取值范围：
                        <br />
                        True（系数非负，可能牺牲模型拟合能力）
                        <br />
                        False（系数可正可负，灵活性高）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入规则最大长度（max_length）">
                <Flex gap={10}>
                  <Form.Item name="max_length" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="规则最大长度"
                    content={
                      <>
                        是否强制回归系数为非负值（适用于需正向解释的场景，如经济学或物理模型）
                        <br />
                        取值范围：
                        <br />
                        True（系数非负，可能牺牲模型拟合能力）
                        <br />
                        False（系数可正可负，灵活性高）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入评估指标（metric）">
                <Flex gap={10}>
                  <Form.Item name="metric" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="评估指标"
                    content={
                      <>
                        是否强制回归系数为非负值（适用于需正向解释的场景，如经济学或物理模型）
                        <br />
                        取值范围：
                        <br />
                        True（系数非负，可能牺牲模型拟合能力）
                        <br />
                        False（系数可正可负，灵活性高）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入最小阈值（min_threshold）">
                <Flex gap={10}>
                  <Form.Item name="min_threshold" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="最小阈值"
                    content={
                      <>
                        是否强制回归系数为非负值（适用于需正向解释的场景，如经济学或物理模型）
                        <br />
                        取值范围：
                        <br />
                        True（系数非负，可能牺牲模型拟合能力）
                        <br />
                        False（系数可正可负，灵活性高）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
            </div>
            <Button type="primary" style={{ height: '360px' }} onClick={handleAISet} loading={isLoading}>
              AI设置
            </Button>
          </Flex>
        </>
      ),
    },
    {
      title: 'Apriori处理',
      content: (
        <>
          <Flex vertical align="center" style={{ marginTop: '20px' }}>
            <Button type="primary" onClick={handleStart} loading={isLoading}>
              开始处理
            </Button>
            <AppProgressSteps current={step} items={[{ title: '启动' }, { title: '计算' }, { title: '计算结束' }]} />
          </Flex>
        </>
      ),
    },
  ]

  const Footer = () => {
    return (
      <Flex justify="center" gap={20}>
        {current === 0 && (
          <Button
            type="primary"
            onClick={async () => {
              await handleAICompute()
              setOpenResults(true)
            }}
            loading={isLoading}>
            AI检查试算
          </Button>
        )}
        {current > 0 && <Button onClick={() => setCurrent(current - 1)}>上一步</Button>}
        {current < steps.length - 1 && (
          <Button type="primary" onClick={handleNext}>
            下一步
          </Button>
        )}
        {current === steps.length - 1 && (
          <Button type="primary" onClick={handleClose} disabled={isLoading}>
            关闭
          </Button>
        )}
      </Flex>
    )
  }

  return (
    <>
      <Button type="primary" onClick={() => setOpen(true)}>
        Apriori
      </Button>
      <Modal
        title={steps[current].title}
        open={open}
        footer={<Footer />}
        destroyOnClose
        width={800}
        onCancel={handleClose}>
        <div style={{ minHeight: 300, padding: '20px 0' }}>
          <Form form={form} preserve={false}>
            {steps.map((item, index) => (
              <div key={index} hidden={current !== index} style={{ marginTop: '20px' }}>
                {item.content}
              </div>
            ))}
          </Form>
          {current === 1 && step >= 3 && (
            <Card style={{ marginTop: '20px' }}>
              <p>计算结束！</p>
              <p>计算结果如下：</p>
              {results.result}
            </Card>
          )}
        </div>
      </Modal>
      <Modal
        title="算法试算检查"
        open={openResults}
        footer={null}
        width={800}
        loading={isLoading}
        onCancel={() => setOpenResults(false)}>
        <p style={{ fontSize: 20, textAlign: 'center' }}>试算检查结果：{results.pass ? '通过' : '不通过'}！</p>
        <MarkdownRender>{code}</MarkdownRender>
      </Modal>
    </>
  )
}
