import { useState } from 'react'
import { Button, Flex, Modal, Select, Form, Input, Popover, Card } from 'antd'
import { QuestionCircleOutlined } from '@ant-design/icons'
import AppProgressSteps from '@/components/AppProgressSteps'
import { useDataFieldQuery } from '@/hooks/useQuery'
import { useAI } from '@/ai/useAI'
import MarkdownRender from '@/components/MarkdownRender'

export default function ButtonModal() {
  const { fieldNameOptions } = useDataFieldQuery()

  const [open, setOpen] = useState(false)
  const [current, setCurrent] = useState(0)
  const [form] = Form.useForm()

  const { post, isLoading } = useAI()

  // AI设置
  const handleAISet = async () => {
    await form.validateFields(['independent_variables', 'dependent_variable'])

    const values = form.getFieldsValue()
    const message = await post({
      response_format: { type: 'json_object' },
      messages: [
        {
          role: 'user',
          content: `根据提供的数据集，设计一个支持向量机算法，返回JSON格式。
          特征列：${values.independent_variables?.join(', ')}
          目标列：${values.dependent_variable}
返回格式示例：
json
        {
          "results": { "c": "正则化参数（C）", "kernel": "核函数类型（kernel）", "gamma": "核函数系数（gamma）", "degree": "多项式核次数（degree）", "class_weight": "类别权重（class_weight）"}
        }
          `,
        },
      ],
    })
    if (message.json && message.json.results) {
      form.setFieldsValue(message.json.results)
    }
  }

  // AI计算
  const [results, setResults] = useState<any>({})
  const code = `${results.code?.replace(/\\n/gi, '\n')}`
  const [openResults, setOpenResults] = useState(false)
  const handleAICompute = async () => {
    const values = await form.validateFields()
    const message = await post({
      response_format: { type: 'json_object' },
      messages: [
        {
          role: 'user',
          content: `根据提供的数据集，设置的支持向量机算法进行试算检查，返回JSON格式。
          特征列：${values.independent_variables?.join(', ')}
          目标列：${values.dependent_variable}
          正则化参数（C）：${values.c}
          核函数类型（kernel）：${values.kernel}
          核函数系数（gamma）：${values.gamma}
          多项式核次数（degree）：${values.degree}
          类别权重（class_weight）：${values.class_weight}
          返回格式示例：
          json
          {
           results: { pass: 'true或者false，代表通过不通过', code: "Python运行的代码，markdown能解析的代码", result: "计算结果为纯文本形式" },
          }
           `,
        },
      ],
    })
    if (message.json && message.json.results) {
      setResults(message.json.results)
    }
    if (current === 1) {
      setStep(3)
    }
  }

  const [step, setStep] = useState<number>(-1)

  const options = [
    { label: 'True', value: true },
    { label: 'False', value: false },
  ]

  // 处理下一步按钮逻辑
  const handleNext = async () => {
    await form.validateFields()
    setCurrent(current + 1)
    setStep(-1)
  }

  // 开始处理
  const handleStart = () => {
    setStep(1)
    handleAICompute()
  }

  // 关闭并重置
  const handleClose = () => {
    setOpen(false)
    setCurrent(0)
    setStep(-1)
  }

  const steps = [
    {
      title: '设置算法',
      content: (
        <>
          <Form.Item label="请选择特征列" name="independent_variables" rules={[{ required: true, message: '请选择' }]}>
            <Select options={fieldNameOptions} placeholder="请选择" mode="multiple" optionFilterProp="label" />
          </Form.Item>
          <Form.Item label="请选择目标列" name="dependent_variable" rules={[{ required: true, message: '请选择' }]}>
            <Select options={fieldNameOptions} placeholder="请选择" />
          </Form.Item>
          <Flex gap={20}>
            <div style={{ flex: 1 }}>
              <Form.Item label="请输入正则化参数（C）">
                <Flex gap={10}>
                  <Form.Item name="c" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="正则化参数（C）"
                    content={
                      <>
                        平衡分类精度和模型复杂度（值越大对误分类惩罚越重）
                        <br />
                        取值范围：&gt;0 的浮点数
                        <br />
                        参数越低意味着允许更多误分类（泛化性强，可能欠拟合），参数越高意味着追求更高分类精度（模型复杂，可能过拟合）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入核函数类型（kernel）">
                <Flex gap={10}>
                  <Form.Item name="kernel" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="核函数类型（kernel）"
                    content={
                      <>
                        决定是否复制输入数据（避免修改原始数据）
                        <br />
                        取值范围：
                        <br />
                        True（安全但内存占用高）
                        <br />
                        False（节省内存，但可能覆盖原始数据）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入核函数系数（gamma）">
                <Flex gap={10}>
                  <Form.Item name="gamma" noStyle rules={[{ required: true, message: '请输入' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="核函数系数（gamma）"
                    content={
                      <>
                        控制核函数的敏感度（影响数据点间的相似性计算）
                        <br />
                        取值范围：&gt;0 的浮点数 或 'scale'（基于特征方差自适应计算）、'auto'（固定为 1/n_features）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入多项式核次数（degree）">
                <Flex gap={10}>
                  <Form.Item name="degree" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Select options={options} placeholder="请选择" />
                  </Form.Item>
                  <Popover
                    title="多项式核次数（degree）"
                    content={
                      <>
                        多项式核的阶数（仅当 kernel='poly' 时生效） <br />
                        取值范围：≥1 的整数 <br />
                        参数越低意味着低阶多项式（决策边界简单，计算快），参数越高意味着高阶多项式（复杂边界，计算成本高，易过拟合）
                        <br />
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入类别权重（class_weight）">
                <Flex gap={10}>
                  <Form.Item name="class_weight" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Select options={options} placeholder="请选择" />
                  </Form.Item>
                  <Popover
                    title="类别权重（class_weight）"
                    content={
                      <>
                        调整不同类别样本的权重（解决类别不平衡问题） 取值范围：
                        <br />
                        None：所有类别权重相等
                        <br />
                        'balanced'：按类别样本数反比自动加权
                        <br />
                        自定义字典（如 {'{'}0:1, 1:5{'}'}）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
            </div>
            <Button type="primary" style={{ height: '250px' }} onClick={handleAISet} loading={isLoading}>
              AI设置
            </Button>
          </Flex>
        </>
      ),
    },
    {
      title: '支持向量机处理',
      content: (
        <>
          <Flex vertical align="center" style={{ marginTop: '20px' }}>
            <Button type="primary" onClick={handleStart} loading={isLoading}>
              开始处理
            </Button>
            <AppProgressSteps current={step} items={[{ title: '启动' }, { title: '计算' }, { title: '计算结束' }]} />
          </Flex>
        </>
      ),
    },
  ]

  const Footer = () => {
    return (
      <Flex justify="center" gap={20}>
        {current === 0 && (
          <Button
            type="primary"
            onClick={async () => {
              await handleAICompute()
              setOpenResults(true)
            }}
            loading={isLoading}>
            AI检查试算
          </Button>
        )}
        {current > 0 && <Button onClick={() => setCurrent(current - 1)}>上一步</Button>}
        {current < steps.length - 1 && (
          <Button type="primary" onClick={handleNext}>
            下一步
          </Button>
        )}
        {current === steps.length - 1 && (
          <Button type="primary" onClick={handleClose} disabled={isLoading}>
            关闭
          </Button>
        )}
      </Flex>
    )
  }

  return (
    <>
      <Button type="primary" onClick={() => setOpen(true)}>
        支持向量机
      </Button>
      <Modal
        title={steps[current].title}
        open={open}
        footer={<Footer />}
        destroyOnClose
        width={800}
        onCancel={handleClose}>
        <div style={{ minHeight: 300, padding: '20px 0' }}>
          <Form form={form} preserve={false}>
            {steps.map((item, index) => (
              <div key={index} hidden={current !== index} style={{ marginTop: '20px' }}>
                {item.content}
              </div>
            ))}
          </Form>
          {current === 1 && step >= 3 && (
            <Card style={{ marginTop: '20px' }}>
              <p>计算结束！</p>
              <p>计算结果如下：</p>
              {results.result}
            </Card>
          )}
        </div>
      </Modal>
      <Modal
        title="算法试算检查"
        open={openResults}
        footer={null}
        width={800}
        loading={isLoading}
        onCancel={() => setOpenResults(false)}>
        <p style={{ fontSize: 20, textAlign: 'center' }}>试算检查结果：{results.pass ? '通过' : '不通过'}！</p>
        <MarkdownRender>{code}</MarkdownRender>
      </Modal>
    </>
  )
}
