import { useState } from 'react'
import { Button, Flex, Modal, Select, Form, Input, Popover, Card } from 'antd'
import { QuestionCircleOutlined } from '@ant-design/icons'
import AppProgressSteps from '@/components/AppProgressSteps'
import { useDataFieldQuery } from '@/hooks/useQuery'
import { useAI } from '@/ai/useAI'
import MarkdownRender from '@/components/MarkdownRender'

export default function ButtonModal() {
  const { fieldNameOptions } = useDataFieldQuery()

  const [open, setOpen] = useState(false)
  const [current, setCurrent] = useState(0)
  const [form] = Form.useForm()

  const { post, isLoading } = useAI()

  // AI设置
  const handleAISet = async () => {
    await form.validateFields(['independent_variables', 'dependent_variable'])

    const values = form.getFieldsValue()
    const message = await post({
      response_format: { type: 'json_object' },
      messages: [
        {
          role: 'user',
          content: `根据提供的数据集，设计一个决策树（二分类）算法，返回JSON格式。
          特征列${values.independent_variables?.join(', ')}
          目标列：${values.dependent_variable}
返回格式示例：
json
        {
          "results": { "max_depth": "树的最大深度（max_depth）", "min_samples_split": "节点分裂最小样本数（min_samples_split）", "min_samples_leaf": "叶节点最小样本数（min_samples_leaf）", "criterion": "分裂标准（criterion）", "max_features": "最大特征数（max_features）"}
        }
          `,
        },
      ],
    })
    console.log(message)
    if (message.json && message.json.results) {
      form.setFieldsValue(message.json.results)
    }
  }

  // AI计算
  const [results, setResults] = useState<any>({})
  const code = `${results.code?.replace(/\\n/gi, '\n')}`
  const [openResults, setOpenResults] = useState(false)
  const handleAICompute = async () => {
    const values = await form.validateFields()
    const message = await post({
      response_format: { type: 'json_object' },
      messages: [
        {
          role: 'user',
          content: `根据提供的数据集，设置的决策树（二分类）算法进行试算检查，返回JSON格式。
          特征列${values.independent_variables?.join(', ')}
          目标列：${values.dependent_variable}
          树的最大深度（max_depth）：${values.max_depth}
          节点分裂最小样本数（min_samples_split）：${values.min_samples_split}
          叶节点最小样本数（min_samples_leaf）：${values.min_samples_leaf}
          分裂标准（criterion）：${values.criterion}
          最大特征数（max_features）：${values.max_features}
          返回格式示例：
          json
          {
           results: { pass: 'true或者false，代表通过不通过', code: "Python运行的代码，markdown能解析的代码", result: "计算结果为纯文本形式" },
          }
           `,
        },
      ],
    })
    if (message.json && message.json.results) {
      setResults(message.json.results)
    }
    if (current === 1) {
      setStep(3)
    }
  }

  const [step, setStep] = useState<number>(-1)

  // 处理下一步按钮逻辑
  const handleNext = async () => {
    await form.validateFields()
    setCurrent(current + 1)
    setStep(-1)
  }

  // 开始处理
  const handleStart = () => {
    setStep(1)
    handleAICompute()
  }

  // 关闭并重置
  const handleClose = () => {
    setOpen(false)
    setCurrent(0)
    setStep(-1)
  }

  const steps = [
    {
      title: '设置算法',
      content: (
        <>
          <Form.Item label="请选择特征列" name="independent_variables" rules={[{ required: true, message: '请选择' }]}>
            <Select options={fieldNameOptions} placeholder="请选择" mode="multiple" optionFilterProp="label" />
          </Form.Item>
          <Form.Item label="请选择目标列" name="dependent_variable" rules={[{ required: true, message: '请选择' }]}>
            <Select options={fieldNameOptions} placeholder="请选择" />
          </Form.Item>
          <Flex gap={20}>
            <div style={{ flex: 1 }}>
              <Form.Item label="请输入树的最大深度（max_depth）">
                <Flex gap={10}>
                  <Form.Item name="max_depth" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="树的最大深度（max_depth）"
                    content={
                      <>
                        限制决策树的最大深度（防止过拟合，控制模型复杂度）
                        <br />
                        取值范围：≥1 的整数 或 None
                        <br />
                        参数越低意味着树浅（模型简单，可能欠拟合），参数越高意味着树深（模型复杂，可能过拟合），None意味着不限制
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入节点分裂最小样本数（min_samples_split）">
                <Flex gap={10}>
                  <Form.Item name="min_samples_split" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="节点分裂最小样本数（min_samples_split）"
                    content={
                      <>
                        定义节点分裂所需的最小样本数（防止过拟合） <br />
                        取值范围：≥2 的整数 或 (0.0, 1.0] 的浮点数（表示样本比例） <br />
                        参数越低意味着分裂限制少（树更复杂，过拟合风险高），参数越高意味着分裂限制多（树更简单，可能欠拟合）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入叶节点最小样本数（min_samples_leaf）">
                <Flex gap={10}>
                  <Form.Item name="min_samples_leaf" noStyle rules={[{ required: true, message: '请输入' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="叶节点最小样本数（min_samples_leaf）"
                    content={
                      <>
                        定义叶节点所需的最小样本数（防止过拟合） <br />
                        取值范围：≥1 的整数 或 (0.0, 0.5] 的浮点数（表示样本比例） <br />
                        参数越低意味着叶节点样本少（对噪声敏感，可能过拟合），参数越高意味着叶节点样本多（泛化性强，可能欠拟合）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入分裂标准（criterion）">
                <Flex gap={10}>
                  <Form.Item name="criterion" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="分裂标准（criterion）"
                    content={
                      <>
                        选择节点分裂的评估指标（影响决策树对特征的选择方式）
                        <br />
                        取值范围：
                        <br />
                        'gini'：基尼不纯度，计算快，适合大多数分类任务
                        <br />
                        'entropy'：信息增益，更敏感于类别分布，适合复杂分类问题
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
              <Form.Item label="请输入最大特征数（max_features）">
                <Flex gap={10}>
                  <Form.Item name="max_features" noStyle rules={[{ required: true, message: '请选择' }]}>
                    <Input placeholder="请输入" />
                  </Form.Item>
                  <Popover
                    title="最大特征数（max_features）"
                    content={
                      <>
                        分裂时考虑的最大特征数（控制特征随机性，提升泛化性） <br />
                        取值范围： <br />
                        整数（固定数量） <br />
                        浮点数（比例） <br />
                        'sqrt'（平方根） <br />
                        'log2'（对数）
                      </>
                    }>
                    <QuestionCircleOutlined />
                  </Popover>
                </Flex>
              </Form.Item>
            </div>
            <Button type="primary" style={{ height: '250px' }} onClick={handleAISet} loading={isLoading}>
              AI设置
            </Button>
          </Flex>
        </>
      ),
    },
    {
      title: '决策树（二分类）处理',
      content: (
        <>
          <Flex vertical align="center" style={{ marginTop: '20px' }}>
            <Button type="primary" onClick={handleStart} loading={isLoading}>
              开始处理
            </Button>
            <AppProgressSteps current={step} items={[{ title: '启动' }, { title: '计算' }, { title: '计算结束' }]} />
          </Flex>
        </>
      ),
    },
  ]

  const Footer = () => {
    return (
      <Flex justify="center" gap={20}>
        {current === 0 && (
          <Button
            type="primary"
            onClick={async () => {
              await handleAICompute()
              setOpenResults(true)
            }}
            loading={isLoading}>
            AI检查试算
          </Button>
        )}
        {current > 0 && <Button onClick={() => setCurrent(current - 1)}>上一步</Button>}
        {current < steps.length - 1 && (
          <Button type="primary" onClick={handleNext}>
            下一步
          </Button>
        )}
        {current === steps.length - 1 && (
          <Button type="primary" onClick={handleClose} disabled={isLoading}>
            关闭
          </Button>
        )}
      </Flex>
    )
  }

  return (
    <>
      <Button type="primary" onClick={() => setOpen(true)}>
        决策树（二分类）
      </Button>
      <Modal
        title={steps[current].title}
        open={open}
        footer={<Footer />}
        destroyOnClose
        width={800}
        onCancel={handleClose}>
        <div style={{ minHeight: 300, padding: '20px 0' }}>
          <Form form={form} preserve={false}>
            {steps.map((item, index) => (
              <div key={index} hidden={current !== index} style={{ marginTop: '20px' }}>
                {item.content}
              </div>
            ))}
          </Form>
          {current === 1 && step >= 3 && (
            <Card style={{ marginTop: '20px' }}>
              <p>计算结束！</p>
              <p>计算结果如下：</p>
              {results.result}
            </Card>
          )}
        </div>
      </Modal>
      <Modal
        title="算法试算检查"
        open={openResults}
        footer={null}
        width={800}
        loading={isLoading}
        onCancel={() => setOpenResults(false)}>
        <p style={{ fontSize: 20, textAlign: 'center' }}>试算检查结果：{results.pass ? '通过' : '不通过'}！</p>
        <MarkdownRender>{code}</MarkdownRender>
      </Modal>
    </>
  )
}
