import { useEffect, useState } from 'react'
import { Button, Flex, Modal, Radio, Input, Form, Row, Col, Select } from 'antd'
import { useDataQuery, useDataFieldQuery } from '@/hooks/useQuery'
import AppProgressSteps from '@/components/AppProgressSteps'
import { useProcessData } from '../query'
import { MinusCircleOutlined } from '@ant-design/icons'
import { uniqBy } from 'lodash-es'
import { useAI } from '@/ai/useAI'
import MarkdownRender from '@/components/MarkdownRender'

export default function ButtonModal() {
  const { data = { list: [] } } = useDataQuery()
  const { fieldOptions, getFieldName } = useDataFieldQuery()
  // 数字类型字段
  const currentFieldOptions = fieldOptions.filter((item) => item.type.includes('VARCHAR'))

  const [open, setOpen] = useState(false)
  const [current, setCurrent] = useState(0)
  const [form] = Form.useForm()
  const [dataSource, setDataSource] = useState<{ key: number; label: string; value: string }[]>([])

  const field = Form.useWatch('field', form)
  const rules = Form.useWatch('rules', form) || []

  useEffect(() => {
    if (field && data.list?.length) {
      const uniqList = uniqBy(data.list, field)
      setDataSource(
        uniqList.map((item: any, index) => ({
          key: index + 1,
          label: item[field] || '',
          value: item[field] || '',
        }))
      )
    }
  }, [field, data.list])

  const [result, setResult] = useState('')
  const { isLoading, post } = useAI()

  const handleAI = async () => {
    const message = await post({
      messages: [
        {
          role: 'user',
          content: `任务目标

分析指定字段的分组规则是否合理，包括 分组逻辑、数据分布、业务规则匹配 ，并提供改进建议。输入要求

字段信息 ：[${getFieldName(field)}]  

数据样本：${JSON.stringify(dataSource)}

分组参数 ：

分组规则 （如"按年龄段分组：0-18、19-65、65+"）。

分组依据 （如"按销售额分组为低/中/高"）。

输出格式：  

   | 序号 | 分组名称  |  当前分组合理性  |  合理性判断依据                   |   改进建议（格式：建议类型：描述，执行方式：自动/人工）

   | 1    | 0-18岁      |  不合理              |    无数据落入此区间              |    建议类型：删除此分组，执行方式：人工

   | 2   | 19-65岁     |  合理                 |    数据分布均匀                    |   无需调整  

   | 1    | 65+岁       |  不合理              |    包含极端值（如年龄300岁） |    建议类型：调整上限为120岁，执行方式：人工

执行要求

合理性评估标准 ：

■
数据覆盖 ：分组是否覆盖所有数据值，无遗漏或重叠。
■
业务匹配 ：是否符合业务需求（如医疗分组需符合年龄段标准）。
■
数据分布均衡性 ：分组内数据量是否合理（如避免空组或数据堆积）。
■
逻辑一致性 ：分组边界是否合理（如"65+岁"是否包含65岁）。`,
        },
      ],
    })
    if (message.content) setResult(message.content)
  }

  // 处理下一步按钮逻辑
  const handleNext = async () => {
    if (current === 0) {
      // 第一步验证表单
      await form.validateFields(['name', 'english_name', 'field'])
    } else if (current === 1) {
      await form.validateFields()
    }
    setCurrent(current + 1)
  }

  const { mutate, isPending, progress, message, remove } = useProcessData()

  // 开始处理
  const handleStart = () => {
    const values = form.getFieldsValue()
    const params = {
      ...values,
      rules: JSON.stringify(
        values.rules.map((item: any) => {
          return {
            ...item,
            fields: item.fields.join(','),
          }
        })
      ),
    }
    mutate(params)
  }

  // 关闭并重置
  const handleClose = () => {
    setOpen(false)
    setCurrent(0)
    remove()
    setResult('')
  }
  const [groupName, setGroupName] = useState('')
  // 步骤定义
  const steps = [
    {
      title: '请选择数据分组字段',
      content: (
        <>
          <Form.Item label="数据分组字段名称" name="name" rules={[{ required: true, message: '请输入字段名称' }]}>
            <Input placeholder="请输入" />
          </Form.Item>
          <Form.Item
            label="数据分组字段英文名称"
            name="english_name"
            rules={[{ required: true, message: '请输入英文名称' }]}>
            <Input placeholder="请输入" />
          </Form.Item>
          <Form.Item
            label="请选择数据分组字段"
            name="field"
            rules={[{ required: true, message: '请选择数据分组字段' }]}>
            <Radio.Group>
              <Row gutter={[10, 10]}>
                {currentFieldOptions.map((item) => (
                  <Col span={8} key={item.value}>
                    <Radio value={item.value}>{item.label}</Radio>
                  </Col>
                ))}
              </Row>
            </Radio.Group>
          </Form.Item>
        </>
      ),
    },
    {
      title: '配置数据分组规则',
      content: (
        <>
          <Form.Item label="数据分组操作字段">{getFieldName(field)}</Form.Item>
          <Form.List name="rules">
            {(fields, { add, remove }) => (
              <>
                <Form.Item label="数据分组名称">
                  <Flex gap={10} align="center">
                    <Input
                      placeholder="请输入"
                      value={groupName}
                      onChange={(e) => setGroupName(e.target.value)}
                      style={{ width: '60%' }}
                    />
                    <Button
                      type="primary"
                      disabled={!groupName}
                      onClick={() => {
                        if (!groupName) return
                        add({ name: groupName, fields: [] }) // 添加新分组
                        setGroupName('')
                      }}>
                      添加分组
                    </Button>
                  </Flex>
                </Form.Item>

                {fields.map(({ key, name, ...restField }) => (
                  <Form.Item {...restField} label={rules[name]?.name} style={{ flex: 1 }} key={key}>
                    <Flex gap={10}>
                      <Form.Item name={[name, 'fields']} noStyle>
                        <Select
                          options={dataSource.filter((item) => {
                            return !rules.some(
                              (rule: any, index: number) => index !== name && rule.fields?.includes(item.value)
                            )
                          })}
                          mode="multiple"
                          allowClear
                          style={{ width: '60%' }}
                        />
                      </Form.Item>
                      <MinusCircleOutlined onClick={() => remove(name)} />
                    </Flex>
                  </Form.Item>
                ))}
              </>
            )}
          </Form.List>
          {result && (
            <div
              style={{
                marginTop: 20,
                background: '#f5f5f5',
                padding: 20,
                maxHeight: 300,
                overflow: 'auto',
                borderRadius: 10,
              }}>
              <MarkdownRender>{result}</MarkdownRender>
            </div>
          )}
        </>
      ),
    },
    {
      title: '处理执行',
      content: (
        <>
          <Flex vertical align="center" style={{ marginTop: '20px' }}>
            <Button type="primary" onClick={handleStart} loading={isPending}>
              开始处理
            </Button>
            <AppProgressSteps
              current={progress}
              items={[
                { title: '新建字段', description: <>{message[1]}</> },
                { title: '复制数据', description: <>{message[2]}</> },
                { title: '数据分组处理', description: <>{message[3]}</> },
                { title: '处理结果', description: <>{message[4]}</> },
              ]}
            />
          </Flex>
        </>
      ),
    },
  ]

  return (
    <>
      <Button type="primary" onClick={() => setOpen(true)}>
        数据分组
      </Button>
      <Modal
        title={steps[current]?.title}
        open={open}
        footer={
          <Flex justify="center" gap={20}>
            {current === 1 && (
              <Button type="primary" onClick={handleAI} loading={isLoading}>
                AI智能建议
              </Button>
            )}
            {current > 0 && <Button onClick={() => setCurrent(current - 1)}>上一步</Button>}
            {current < steps.length - 1 && (
              <Button type="primary" onClick={handleNext}>
                下一步
              </Button>
            )}
            {current === steps.length - 1 && (
              <Button type="primary" onClick={handleClose} disabled={isPending}>
                关闭
              </Button>
            )}
          </Flex>
        }
        destroyOnClose
        width={800}
        onCancel={handleClose}>
        <div style={{ minHeight: 300, padding: '20px 0' }}>
          <Form form={form} labelCol={{ span: 5 }} preserve={false}>
            {steps.map((item, index) => (
              <div key={index} hidden={current !== index} style={{ marginTop: '20px' }}>
                {item.content}
              </div>
            ))}
          </Form>
        </div>
      </Modal>
    </>
  )
}
